from django.shortcuts import render
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse

# Create your views here.
from .models import ProductPage,ProductAttributeValue,Category,Brands,ProductVariantPage,ProductVariantAttributeValue
from wagtail.search.utils import parse_query_string

from wagtail.models import Page
from wagtail.contrib.search_promotions.models import Query


def search(request):
    # In [33]: Page.objects.live().search("4GB 12GB 5G Smartphone",operator="or")
    query=request.GET['q']
    query=query.replace('k','000')
    # query=query.replace('+','')
    # query=query.replace('-',' ').split()
    # print("Val=",query.split())
    q=query
    # prodObj=ProductPage.objects.filter(prod_attr_values__value_text__in=q,tags__name__in=q)
    print("Search=",q)
    
    
    if q:
        search_results = Page.objects.live().search(q,operator="or")
        Query.get(q).add_hit()
    else:
        search_results = Page.objects.none()
   
    print("search_results=",search_results)
    return TemplateResponse(request,'Pages/filter.html',{'prodObj':search_results,'title':request.GET['q']})
    
    # return render(request,'Pages/search_result.html',{'prodObj':prodObj,'title':request.GET['q']})


def story(request, slug):
    if ProductPage.objects.filter(slug=slug).exists():
        search_results = ProductPage.objects.get(slug=slug)
    return render(request, 'Pages/story_page.html', {
        'search_results': search_results,
    })

def compare_products(request):
    total=request.GET['total']
    prodList=[]
    prodAttrs=[]
    for i in range(0,int(total)):
        prodList.append(ProductVariantPage.objects.get(title=request.GET['prod'+str(i)]))
    for pa in prodList:
        for attr in ProductVariantAttributeValue.objects.filter(product=pa):
            if attr.attribute.attr:
                prodAttrs.append(attr.attribute.attr.name)
    return render(request,'Pages/compare.html',{'prodList':prodList,'prodAttrs':list(dict.fromkeys(prodAttrs))
})


### =========Chooser View Sets=============

from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Q
from django.utils.translation import gettext_lazy as _
from wagtail.admin.forms.choosers import BaseFilterForm, LocaleFilterMixin
from wagtail.admin.ui.tables import TitleColumn
from wagtail.admin.views.generic.chooser import (BaseChooseView,
                                                 ChooseResultsViewMixin,
                                                 ChooseViewMixin,
                                                 CreationFormMixin)
from wagtail.admin.viewsets.chooser import ChooserViewSet
from wagtail.models import TranslatableMixin


class CategorySearchFilterMixin(forms.Form):
    q = forms.CharField(
        label=_("Search Category"),
        widget=forms.TextInput(attrs={"placeholder": _("Search")}),
        required=False,
    )
    def filter(self, objects):
        objects = super().filter(objects)
        search_query = self.cleaned_data.get("q")
        if search_query:
            objects = objects.filter(
                Q(name__icontains=search_query) |
                Q(slug__icontains=search_query)
            )
            self.is_searching = True
            self.search_query = search_query
        return objects

class BaseCategoryChooseView(BaseChooseView):
    ordering = ["name", "slug"]
    @property
    def columns(self):
        return [
            TitleColumn(
                name="name",
                label=_("Category"),
                accessor="name",
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="slug",
                label=_("Slug"),
                accessor="slug",
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
        ]

    def get_filter_form_class(self):
        bases = [CategorySearchFilterMixin, BaseFilterForm]

        i18n_enabled = getattr(settings, "WAGTAIL_I18N_ENABLED", False)
        if i18n_enabled and issubclass(self.model_class, TranslatableMixin):
            bases.insert(0, LocaleFilterMixin)

        return type(
            "FilterForm",
            tuple(bases),
            {},
        )

class CategoryChooseView(ChooseViewMixin, CreationFormMixin,  BaseCategoryChooseView):
    pass

class CategoryChooseResultsView(
    ChooseResultsViewMixin, CreationFormMixin, BaseCategoryChooseView
):
    pass
class CategoryChooserViewSet(ChooserViewSet):
    model = Category
    choose_view_class = CategoryChooseView
    choose_results_view_class = CategoryChooseResultsView
    icon = "list"
    choose_one_text = _("Choose a category")
    choose_another_text = _("Choose another category")
    edit_item_text = _("Edit this category")

category_chooser_viewset = CategoryChooserViewSet("category_chooser")




class BrandSearchFilterMixin(forms.Form):
    q = forms.CharField(
        label=_("Search Brand"),
        widget=forms.TextInput(attrs={"placeholder": _("Search")}),
        required=False,
    )
    def filter(self, objects):
        objects = super().filter(objects)
        search_query = self.cleaned_data.get("q")
        if search_query:
            objects = objects.filter(
                Q(name__icontains=search_query) |
                Q(slug__icontains=search_query)|
                Q(com__coName__icontains=search_query)
            )
            self.is_searching = True
            self.search_query = search_query
        return objects

class BaseBrandChooseView(BaseChooseView):
    ordering = ["name", "slug"]
    @property
    def columns(self):
        return [
            TitleColumn(
                name="name",
                label=_("Brand"),
                accessor="name",
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
            TitleColumn(
                name="com",
                label=_("Company"),
                accessor="com",
                url_name=self.chosen_url_name,
                link_attrs={"data-chooser-modal-choice": True},
            ),
        ]

    def get_filter_form_class(self):
        bases = [CategorySearchFilterMixin, BaseFilterForm]

        i18n_enabled = getattr(settings, "WAGTAIL_I18N_ENABLED", False)
        if i18n_enabled and issubclass(self.model_class, TranslatableMixin):
            bases.insert(0, LocaleFilterMixin)

        return type(
            "FilterForm",
            tuple(bases),
            {},
        )

class BrandChooseView(ChooseViewMixin, CreationFormMixin,  BaseBrandChooseView):
    pass

class BrandChooseResultsView(
    ChooseResultsViewMixin, CreationFormMixin, BaseBrandChooseView
):
    pass
class BrandChooserViewSet(ChooserViewSet):
    model = Brands
    choose_view_class = BrandChooseView
    choose_results_view_class = BrandChooseResultsView
    icon = "list"
    choose_one_text = _("Choose a brand")
    choose_another_text = _("Choose another brand")
    edit_item_text = _("Edit this brand")

brand_chooser_viewset = BrandChooserViewSet("brand_chooser")